@@ -103,8 +103,7 @@ L.Editable = L.Class.extend({
         if (L.Browser.touch) this.map.off('click', editor.onTouch, editor);
         if (editor !== this._drawingEditor) return;
         delete this._drawingEditor;
-        this.map.fire('editable:unregisterededitor', {editor: editor});
-        if (editor.drawing) editor.finishDrawing();
+        if (editor.drawing) editor.cancelDrawing();
         L.DomUtil.removeClass(this.map._container, this.options.drawingCSSClass);
     },
 
@@ -135,7 +134,6 @@ L.Editable = L.Class.extend({
         multi.addLayer(polygon);
         polygon.multi = multi;
         var editor = polygon.enableEdit();
-        multi.setPrimary(polygon);
         editor.startDrawingForward();
         return polygon;
     },
@@ -210,19 +208,10 @@ L.Editable.VertexMarker = L.Marker.extend({
         this.editor = editor;
         L.Marker.prototype.initialize.call(this, latlng, options);
         this.options.icon = this.editor.tools.createVertexIcon({className: this.options.className});
-        if (this.editor.secondary) this.setSecondary();
         this.latlng.__vertex = this;
         this.editor.editLayer.addLayer(this);
     },
 
-    setSecondary: function () {
-        this.setOpacity(0.3);
-    },
-
-    setPrimary: function () {
-        this.setOpacity(1);
-    },
-
     onAdd: function (map) {
         L.Marker.prototype.onAdd.call(this, map);
         L.DomEvent.on(this.dragging._draggable, 'drag', this.onDrag, this);
@@ -248,33 +237,35 @@ L.Editable.VertexMarker = L.Marker.extend({
     },
 
     onClick: function (e) {
-        this.editor.onVertexMarkerClick(e, this);
+        e.vertex = this;
+        this.editor.onVertexMarkerClick(e);
     },
 
     onContextMenu: function (e) {
-        this.editor.onVertexMarkerContextMenu(e, this);
+        e.vertex = this;
+        this.editor.onVertexMarkerContextMenu(e);
     },
 
     onMouseDown: function (e) {
-        if (this.editor.secondary) {
-            this.editor.setPrimary();
-        }
+        e.vertex = this;
+        this.editor.onVertexMarkerMouseDown(e);
     },
 
-    remove: function () {
+    delete: function () {
         var next = this.getNext();  // Compute before changing latlng
         this.latlngs.splice(this.latlngs.indexOf(this.latlng), 1);
         this.editor.editLayer.removeLayer(this);
+        this.editor.onVertexDeleted({latlng: this.latlng, vertex: this});
         if (next) next.resetMiddleMarker();
     },
 
     onRemove: function (map) {
-        if (this.middleMarker) this.middleMarker.remove();
+        if (this.middleMarker) this.middleMarker.delete();
         delete this.latlng.__vertex;
         L.Marker.prototype.onRemove.call(this, map);
     },
 
-    getPosition: function () {
+    getIndex: function () {
         return this.latlngs.indexOf(this.latlng);
     },
 
@@ -284,19 +275,19 @@ L.Editable.VertexMarker = L.Marker.extend({
 
     getPrevious: function () {
         if (this.latlngs.length < 2) return;
-        var position = this.getPosition(),
-            previousPosition = position - 1;
-        if (position === 0 && this.editor.CLOSED) previousPosition = this.getLastIndex();
-        var previous = this.latlngs[previousPosition];
+        var index = this.getIndex(),
+            previousIndex = index - 1;
+        if (index === 0 && this.editor.CLOSED) previousIndex = this.getLastIndex();
+        var previous = this.latlngs[previousIndex];
         if (previous) return previous.__vertex;
     },
 
     getNext: function () {
         if (this.latlngs.length < 2) return;
-        var position = this.getPosition(),
-            nextPosition = position + 1;
-        if (position === this.getLastIndex() && this.editor.CLOSED) nextPosition = 0;
-        var next = this.latlngs[nextPosition];
+        var index = this.getIndex(),
+            nextIndex = index + 1;
+        if (index === this.getLastIndex() && this.editor.CLOSED) nextIndex = 0;
+        var next = this.latlngs[nextIndex];
         if (next) return next.__vertex;
     },
 
@@ -317,7 +308,7 @@ L.Editable.VertexMarker = L.Marker.extend({
     },
 
     resetMiddleMarker: function () {
-        if (this.middleMarker) this.middleMarker.remove();
+        if (this.middleMarker) this.middleMarker.delete();
         this.addMiddleMarker();
     },
 
@@ -347,18 +338,9 @@ L.Editable.MiddleMarker = L.Marker.extend({
         this.latlngs = latlngs;
         L.Marker.prototype.initialize.call(this, this.computeLatLng(), options);
         this.options.icon = this.editor.tools.createVertexIcon({className: this.options.className});
-        if (this.editor.secondary) this.setSecondary();
         this.editor.editLayer.addLayer(this);
     },
 
-    setSecondary: function () {
-        this.setOpacity(0.2);
-    },
-
-    setPrimary: function () {
-        this.setOpacity(this.options.opacity);
-    },
-
     updateLatLng: function () {
         this.setLatLng(this.computeLatLng());
     },
@@ -375,15 +357,15 @@ L.Editable.MiddleMarker = L.Marker.extend({
     },
 
     onMouseDown: function (e) {
+        this.editor.onMiddleMarkerMouseDown(e, this);
         this.latlngs.splice(this.index(), 0, e.latlng);
         this.editor.refresh();
-        this.editor.setPrimary();
         var marker = this.editor.addVertexMarker(e.latlng, this.latlngs);
         marker.dragging._draggable._onDown(e.originalEvent);  // Transfer ongoing dragging to real marker
-        this.remove();
+        this.delete();
     },
 
-    remove: function () {
+    delete: function () {
         this.editor.editLayer.removeLayer(this);
     },
 
@@ -430,31 +412,44 @@ L.Editable.BaseEditor = L.Class.extend({
         this.tools.editLayer.removeLayer(this.editLayer);
         this.onDisable();
         delete this._enabled;
+        if (this.drawing) this.cancelDrawing();
         return this;
     },
 
+    _fireAndForward: function (type, e) {
+        e = e || {};
+        e.layer = this.feature;
+        this.feature.fire(type, e);
+        if (this.feature.multi) this.feature.multi.fire(type, e);
+        this.map.fire(type, e);
+    },
+
     onEnable: function () {
-        this.map.fire('editable:enable', {layer: this.feature});
+        this._fireAndForward('editable:enable');
     },
 
     onDisable: function () {
-        this.map.fire('editable:disable', {layer: this.feature});
+        this._fireAndForward('editable:disable');
     },
 
     onEditing: function () {
-        this.map.fire('editable:editing', {layer: this.feature});
-    },
-
-    onEdited: function () {
-        this.map.fire('editable:edited', {layer: this.feature});
+        this._fireAndForward('editable:editing');
     },
 
     onStartDrawing: function () {
-        this.map.fire('editable:startdrawing', {layer: this.feature});
+        this._fireAndForward('editable:drawing:start');
+    },
+
+    onEndDrawing: function () {
+        this._fireAndForward('editable:drawing:end');
+    },
+
+    onCancelDrawing: function () {
+        this._fireAndForward('editable:drawing:cancel');
     },
 
     onFinishDrawing: function () {
-        this.map.fire('editable:enddrawing', {layer: this.feature});
+        this._fireAndForward('editable:drawing:finish');
     },
 
     startDrawing: function () {
@@ -464,10 +459,19 @@ L.Editable.BaseEditor = L.Class.extend({
     },
 
     finishDrawing: function () {
-        this.onEdited();
+        this.endDrawing();
+        this.onFinishDrawing();
+    },
+
+    cancelDrawing: function () {
+        this.endDrawing();
+        this.onCancelDrawing();
+    },
+
+    endDrawing: function () {
         this.drawing = false;
         this.tools.unregisterForDrawing(this);
-        this.onFinishDrawing();
+        this.onEndDrawing();
     },
 
     onMouseMove: function (e) {
@@ -479,6 +483,16 @@ L.Editable.BaseEditor = L.Class.extend({
     onTouch: function (e) {
         this.onMouseMove(e);
         if (this.drawing) this.tools.newClickHandler._fireMouseEvent(e);
+    },
+
+    onNewClickHandlerClicked: function (e) {
+        if (!this.isNewClickValid(e.latlng)) return;
+        this._fireAndForward('editable:drawing:click', e);
+        return true;
+    },
+
+    isNewClickValid: function (latlng) {
+        return true;
     }
 
 });
@@ -508,9 +522,7 @@ L.Editable.MarkerEditor = L.Editable.BaseEditor.extend({
     },
 
     onNewClickHandlerClicked: function (e) {
-        if (this.checkAddConstraints && !this.checkAddConstraints(e.latlng)) {
-            return;
-        }
+        if (!L.Editable.BaseEditor.prototype.onNewClickHandlerClicked.call(this, e)) return;
         this.finishDrawing();
     }
 
@@ -521,9 +533,8 @@ L.Editable.PathEditor = L.Editable.BaseEditor.extend({
     CLOSED: false,
     MIN_VERTEX: 2,
 
-    enable: function (secondary) {
+    enable: function () {
         L.Editable.BaseEditor.prototype.enable.call(this);
-        this.secondary = secondary;
         if (this.feature) {
             this.initVertexMarkers();
         }
@@ -534,23 +545,6 @@ L.Editable.PathEditor = L.Editable.BaseEditor.extend({
         L.Editable.BaseEditor.prototype.disable.call(this);
     },
 
-    setPrimary: function () {
-        if (this.feature.multi) {
-            this.feature.multi.setSecondary(this.feature);
-        }
-        delete this.secondary;
-        this.editLayer.eachLayer(function (layer) {
-            layer.setPrimary();
-        });
-    },
-
-    setSecondary: function () {
-        this.secondary = true;
-        this.editLayer.eachLayer(function (layer) {
-            if (layer.setSecondary) layer.setSecondary();
-        });
-    },
-
     initVertexMarkers: function () {
         // groups can be only latlngs (for polyline or symple polygon,
         // or latlngs plus many holes, in case of a complex polygon)
@@ -560,6 +554,14 @@ L.Editable.PathEditor = L.Editable.BaseEditor.extend({
         }
     },
 
+    getLatLngsGroups: function () {
+        return [this.getLatLngs()];
+    },
+
+    getLatLngs: function () {
+        return this.feature.getLatLngs();
+    },
+
     reset: function () {
         this.editLayer.clearLayers();
         this.initVertexMarkers();
@@ -580,60 +582,65 @@ L.Editable.PathEditor = L.Editable.BaseEditor.extend({
     },
 
     onVertexMarkerClick: function (e, vertex) {
-        var position = vertex.getPosition();
+        var index = e.vertex.getIndex();
         if (e.originalEvent.ctrlKey) {
-            this.onVertexMarkerCtrlClick(e, vertex, position);
+            this.onVertexMarkerCtrlClick(e);
         } else if (e.originalEvent.altKey) {
-            this.onVertexMarkerAltClick(e, vertex, position);
+            this.onVertexMarkerAltClick(e);
         } else if (e.originalEvent.shiftKey) {
-            this.onVertexMarkerShiftClick(e, vertex, position);
-        } else if (position >= 1 && position === vertex.getLastIndex() && this.drawing === L.Editable.FORWARD) {
+            this.onVertexMarkerShiftClick(e);
+        } else if (index >= 1 && index === e.vertex.getLastIndex() && this.drawing === L.Editable.FORWARD) {
             this.finishDrawing();
-        } else if (position === 0 && this.drawing === L.Editable.BACKWARD && this.activeLatLngs.length >= this.MIN_VERTEX) {
+        } else if (index === 0 && this.drawing === L.Editable.BACKWARD && this._drawnLatLngs.length >= this.MIN_VERTEX) {
             this.finishDrawing();
-        } else if (position === 0 && this.drawing === L.Editable.FORWARD && this.activeLatLngs.length >= this.MIN_VERTEX && this.CLOSED) {
+        } else if (index === 0 && this.drawing === L.Editable.FORWARD && this._drawnLatLngs.length >= this.MIN_VERTEX && this.CLOSED) {
             this.finishDrawing();  // Allow to close on first point also for polygons
         } else {
-            this.onVertexRawMarkerClick(e, vertex, position);
+            this.onVertexRawMarkerClick(e);
         }
     },
 
-    onVertexRawMarkerClick: function (e, vertex, position) {
-        if (!this.vertexCanBeRemoved(vertex, position)) return;
-        vertex.remove();
+    onVertexRawMarkerClick: function (e) {
+        if (!this.vertexCanBeDeleted(e.vertex)) return;
+        e.vertex.delete();
         this.refresh();
     },
 
-    vertexCanBeRemoved: function (vertex, position) {
+    vertexCanBeDeleted: function (vertex) {
         return vertex.latlngs.length > this.MIN_VERTEX;
     },
 
-    _fireVertexMarkerEvent: function (type, e, vertex, position) {
-        var event = {
-            originalEvent: e.originalEvent,
-            latlng: e.latlng,
-            vertex: vertex,
-            position: position,
-            layer: this.feature
-        };
-        this.feature.fire(type, event);
-        this.map.fire(type, event);
+    onVertexDeleted: function (e) {
+        this._fireAndForward('editable:vertex:deleted', e);
     },
 
-    onVertexMarkerCtrlClick: function (e, vertex, position) {
-        this._fireVertexMarkerEvent('editable:vertex:ctrlclick', e, vertex, position);
+    onVertexMarkerCtrlClick: function (e) {
+        this._fireAndForward('editable:vertex:ctrlclick', e);
     },
 
-    onVertexMarkerShiftClick: function (e, vertex, position) {
-        this._fireVertexMarkerEvent('editable:vertex:shiftclick', e, vertex, position);
+    onVertexMarkerShiftClick: function (e) {
+        this._fireAndForward('editable:vertex:shiftclick', e);
     },
 
-    onVertexMarkerAltClick: function (e, vertex, position) {
-        this._fireVertexMarkerEvent('editable:vertex:altclick', e, vertex, position);
+    onVertexMarkerAltClick: function (e) {
+        this._fireAndForward('editable:vertex:altclick', e);
     },
 
-    onVertexMarkerContextMenu: function (e, vertex) {
-        this._fireVertexMarkerEvent('editable:vertex:contextmenu', e, vertex, vertex.getPosition());
+    onVertexMarkerContextMenu: function (e) {
+        this._fireAndForward('editable:vertex:contextmenu', e);
+    },
+
+    onVertexMarkerMouseDown: function (e) {
+        this._fireAndForward('editable:vertex:mousedown', e);
+    },
+
+    onMiddleMarkerMouseDown: function (e) {
+        this._fireAndForward('editable:middlemarker:mousedown', e);
+    },
+
+    startDrawing: function () {
+        if (!this._drawnLatLngs) this._drawnLatLngs = this.getLatLngs();
+        L.Editable.BaseEditor.prototype.startDrawing.call(this);
     },
 
     startDrawingForward: function () {
@@ -641,20 +648,18 @@ L.Editable.PathEditor = L.Editable.BaseEditor.extend({
         this.tools.attachForwardLineGuide();
     },
 
-    finishDrawing: function () {
-        L.Editable.BaseEditor.prototype.finishDrawing.call(this);
+    endDrawing: function () {
+        L.Editable.BaseEditor.prototype.endDrawing.call(this);
         this.tools.detachForwardLineGuide();
         this.tools.detachBackwardLineGuide();
-        this.unsetActiveLatLngs();
-        delete this.checkConstraints;
+        delete this._drawnLatLngs;
     },
 
     addLatLng: function (latlng) {
-        this.setActiveLatLngs(latlng);
-        if (this.drawing === L.Editable.FORWARD) this.activeLatLngs.push(latlng);
-        else this.activeLatLngs.unshift(latlng);
+        if (this.drawing === L.Editable.FORWARD) this._drawnLatLngs.push(latlng);
+        else this._drawnLatLngs.unshift(latlng);
         this.refresh();
-        this.addVertexMarker(latlng, this.activeLatLngs);
+        this.addVertexMarker(latlng, this._drawnLatLngs);
     },
 
     newPointForward: function (latlng) {
@@ -671,22 +676,9 @@ L.Editable.PathEditor = L.Editable.BaseEditor.extend({
     },
 
     onNewClickHandlerClicked: function (e) {
-        if (this.checkAddConstraints && !this.checkAddConstraints(e.latlng)) {
-            return;
-        }
+        if (!L.Editable.BaseEditor.prototype.onNewClickHandlerClicked.call(this, e)) return;
         if (this.drawing === L.Editable.FORWARD) this.newPointForward(e.latlng);
         else this.newPointBackward(e.latlng);
-        this.feature.fire('editable:newclick', e);
-    },
-
-    setActiveLatLngs: function (latlng) {
-        if (!this.activeLatLngs) {
-            this.activeLatLngs = this.getLatLngs(latlng);
-        }        
-    },
-
-    unsetActiveLatLngs: function () {
-        delete this.activeLatLngs;
     },
 
     onMouseMove: function (e) {
@@ -706,14 +698,6 @@ L.Editable.PathEditor = L.Editable.BaseEditor.extend({
 
 L.Editable.PolylineEditor = L.Editable.PathEditor.extend({
 
-    getLatLngsGroups: function () {
-        return [this.getLatLngs()];
-    },
-
-    getLatLngs: function () {
-        return this.feature.getLatLngs();
-    },
-
     startDrawingBackward: function () {
         this.drawing = L.Editable.BACKWARD;
         this.startDrawing();
@@ -746,7 +730,7 @@ L.Editable.PolygonEditor = L.Editable.PathEditor.extend({
     MIN_VERTEX: 3,
 
     getLatLngsGroups: function () {
-        var groups = [this.feature._latlngs];
+        var groups = L.Editable.PathEditor.prototype.getLatLngsGroups.call(this);
         if (this.feature._holes) {
             for (var i = 0; i < this.feature._holes.length; i++) {
                 groups.push(this.feature._holes[i]);
@@ -760,21 +744,6 @@ L.Editable.PolygonEditor = L.Editable.PathEditor.extend({
         this.tools.attachBackwardLineGuide();
     },
 
-    finishDrawing: function () {
-        L.Editable.PathEditor.prototype.finishDrawing.call(this);
-        this.tools.detachBackwardLineGuide();
-    },
-
-    getLatLngs: function (latlng) {
-        if (latlng) {
-            var p = this.map.latLngToLayerPoint(latlng);
-            if (this.feature._latlngs && this.feature._holes && this.feature._containsPoint(p)) {
-                return this.addNewEmptyHole();
-            }
-        }
-        return this.feature._latlngs;
-    },
-
     addNewEmptyHole: function () {
         var holes = Array();
         if (!this.feature._holes) {
@@ -784,13 +753,8 @@ L.Editable.PolygonEditor = L.Editable.PathEditor.extend({
         return holes;
     },
 
-    prepareForNewHole: function () {
-        this.activeLatLngs = this.addNewEmptyHole();
-        this.checkAddConstraints = this.checkContains;
-    },
-
     newHole: function (latlng) {
-        this.prepareForNewHole();
+        this._drawnLatLngs = this.addNewEmptyHole();
         this.startDrawingForward();
         if (latlng) this.newPointForward(latlng);
     },
@@ -799,11 +763,22 @@ L.Editable.PolygonEditor = L.Editable.PathEditor.extend({
         return this.feature._containsPoint(this.map.latLngToLayerPoint(latlng));
     },
 
-    vertexCanBeRemoved: function (vertex) {
-        if (vertex.latlngs === this.feature._latlngs) return L.Editable.PathEditor.prototype.vertexCanBeRemoved.call(this, vertex);
-        else return true;  // Holes can be totally removed without removing the layer itself
+    vertexCanBeDeleted: function (vertex) {
+        if (vertex.latlngs === this.getLatLngs()) return L.Editable.PathEditor.prototype.vertexCanBeDeleted.call(this, vertex);
+        else return true;  // Holes can be totally deleted without removing the layer itself
     },
 
+    isNewClickValid: function (latlng) {
+        if (this._drawnLatLngs !== this.getLatLngs()) return this.checkContains(latlng);
+        return true;
+    },
+
+    onVertexDeleted: function (e) {
+        L.Editable.PathEditor.prototype.onVertexDeleted.call(this, e);
+        if (!e.vertex.latlngs.length && e.vertex.latlngs !== this.getLatLngs()) {
+            this.feature._holes.splice(this.feature._holes.indexOf(e.vertex.latlngs), 1);
+        }
+    }
 
 });
 
@@ -827,12 +802,12 @@ var EditableMixin = {
         return new Klass(map, this);
     },
 
-    enableEdit: function (secondary) {
+    enableEdit: function () {
         if (!this.editor) {
             this.createEditor();
             this.on('remove', this.disableEdit);
         }
-        return this.editor.enable(secondary);
+        return this.editor.enable();
     },
 
     editEnabled: function () {
@@ -950,8 +925,7 @@ var MultiEditableMixin = {
     enableEdit: function (e) {
         this.eachLayer(function(layer) {
             layer.multi = this;
-            layer.disableEdit();
-            layer.enableEdit(!e || e.layer !== layer);
+            layer.enableEdit();
         }, this);
     },
 
@@ -962,24 +936,11 @@ var MultiEditableMixin = {
     },
 
     toggleEdit: function (e) {
-        if (!e.layer.editor || e.layer.editor.secondary) {
+        if (!e.layer.editor) {
             this.enableEdit(e);
         } else {
             this.disableEdit();
         }
-    },
-
-    setPrimary: function (primary) {
-        this.eachLayer(function (layer) {
-            if (layer === primary) layer.editor.setPrimary();
-            else layer.editor.setSecondary();
-        });
-    },
-
-    setSecondary: function (except) {
-        this.eachLayer(function (layer) {
-            if (layer !== except) layer.editor.setSecondary();
-        });
     }
 
 };
